# Generated by roxygen2: do not edit by hand

S3method(VarCorr,glmmTMB)
S3method(anova,glmmTMB)
S3method(as.data.frame,ranef.glmmTMB)
S3method(bread,glmmTMB)
S3method(coef,glmmTMB)
S3method(confint,glmmTMB)
S3method(confint,profile.glmmTMB)
S3method(deviance,glmmTMB)
S3method(df.residual,glmmTMB)
S3method(estfun,glmmTMB)
S3method(extractAIC,glmmTMB)
S3method(family,glmmTMB)
S3method(fitted,glmmTMB)
S3method(fixef,glmmTMB)
S3method(formula,glmmTMB)
S3method(getGroups,glmmTMB)
S3method(getME,glmmTMB)
S3method(has.intercept,glmmTMB)
S3method(isLMM,glmmTMB)
S3method(logLik,glmmTMB)
S3method(meatHC,default)
S3method(meatHC,glmmTMB)
S3method(model.frame,glmmTMB)
S3method(model.matrix,glmmTMB)
S3method(ngrps,default)
S3method(ngrps,factor)
S3method(ngrps,glmmTMB)
S3method(nobs,glmmTMB)
S3method(predict,glmmTMB)
S3method(print,VarCorr.glmmTMB)
S3method(print,coef.glmmTMB)
S3method(print,fixef.glmmTMB)
S3method(print,glmmTMB)
S3method(print,ranef.glmmTMB)
S3method(print,summary.glmmTMB)
S3method(print,vcov.glmmTMB)
S3method(profile,glmmTMB)
S3method(ranef,glmmTMB)
S3method(refit,glmmTMB)
S3method(residuals,glmmTMB)
S3method(sandwich,default)
S3method(sandwich,glmmTMB)
S3method(sigma,glmmTMB)
S3method(simulate,glmmTMB)
S3method(summary,glmmTMB)
S3method(terms,glmmTMB)
S3method(vcov,glmmTMB)
S3method(vcovHC,glmmTMB)
S3method(weights,glmmTMB)
export(VarCorr)
export(bell)
export(beta_family)
export(betabinomial)
export(bread)
export(compois)
export(diagnose)
export(dof_KR)
export(dof_satt)
export(dtruncated_nbinom1)
export(dtruncated_nbinom2)
export(dtruncated_poisson)
export(estfun)
export(family_params)
export(finalizeTMB)
export(fitTMB)
export(fixef)
export(genpois)
export(getCapabilities)
export(getGroups)
export(getGrpVar)
export(getME)
export(getReStruc)
export(get_cor)
export(glmmTMB)
export(glmmTMBControl)
export(gt_load)
export(lognormal)
export(meatHC)
export(nbinom1)
export(nbinom12)
export(nbinom2)
export(numFactor)
export(omp_check)
export(ordbeta)
export(parseNumLevels)
export(put_cor)
export(ranef)
export(refit)
export(sandwich)
export(set_simcodes)
export(sigma)
export(simulate_new)
export(skewnormal)
export(t_family)
export(truncated_compois)
export(truncated_genpois)
export(truncated_nbinom1)
export(truncated_nbinom2)
export(truncated_poisson)
export(tweedie)
export(up2date)
export(vcovHC)
export(ziGamma)
if(getRversion() >= "3.6.0") { S3method(car::Anova, glmmTMB) } else { export(Anova.glmmTMB) }
if(getRversion() >= "3.6.0") { S3method(effects::Effect, glmmTMB)  } else { export(Effect.glmmTMB) }
if(getRversion() >= "3.6.0") { S3method(multcomp::modelparm, glmmTMB) } else {  export(modelparm.glmmTMB) }
if(getRversion()>='3.3.0') { importFrom(stats, sigma) } else { importFrom(lme4,sigma) }
importFrom(Matrix,Cholesky)
importFrom(Matrix,crossprod)
importFrom(Matrix,diag)
importFrom(Matrix,qr)
importFrom(Matrix,qr2rankMatrix)
importFrom(Matrix,rankMatrix)
importFrom(Matrix,solve)
importFrom(Matrix,t)
importFrom(TMB,MakeADFun)
importFrom(TMB,sdreport)
importFrom(TMB,tmbprofile)
importFrom(lme4,.prt.VC)
importFrom(lme4,.prt.aictab)
importFrom(lme4,.prt.call)
importFrom(lme4,.prt.family)
importFrom(lme4,.prt.grps)
importFrom(lme4,.prt.resids)
importFrom(lme4,getME)
importFrom(lme4,isLMM)
importFrom(lme4,refit)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(mgcv,PredictMat)
importFrom(mgcv,s)
importFrom(mgcv,smooth2random)
importFrom(mgcv,smoothCon)
importFrom(nlme,VarCorr)
importFrom(nlme,fixef)
importFrom(nlme,getGroups)
importFrom(nlme,ranef)
importFrom(numDeriv,hessian)
importFrom(numDeriv,jacobian)
importFrom(reformulas,"RHSForm<-")
importFrom(reformulas,RHSForm)
importFrom(reformulas,addForm)
importFrom(reformulas,addForm0)
importFrom(reformulas,anySpecial)
importFrom(reformulas,extractForm)
importFrom(reformulas,findbars)
importFrom(reformulas,findbars_x)
importFrom(reformulas,formatVC)
importFrom(reformulas,inForm)
importFrom(reformulas,makeOp)
importFrom(reformulas,mkReTrms)
importFrom(reformulas,noSpecials)
importFrom(reformulas,no_specials)
importFrom(reformulas,nobars)
importFrom(reformulas,reOnly)
importFrom(reformulas,splitForm)
importFrom(reformulas,sub_specials)
importFrom(reformulas,subbars)
importFrom(sandwich,bread)
importFrom(sandwich,estfun)
importFrom(sandwich,vcovHC)
importFrom(splines,backSpline)
importFrom(splines,interpSpline)
importFrom(stats,"contrasts<-")
importFrom(stats,.getXlevels)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,anova)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,contrasts)
importFrom(stats,cov2cor)
importFrom(stats,delete.response)
importFrom(stats,df.residual)
importFrom(stats,dist)
importFrom(stats,dnbinom)
importFrom(stats,dpois)
importFrom(stats,family)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,getCall)
importFrom(stats,logLik)
importFrom(stats,make.link)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,napredict)
importFrom(stats,nlminb)
importFrom(stats,nobs)
importFrom(stats,optimHess)
importFrom(stats,pbinom)
importFrom(stats,pchisq)
importFrom(stats,plogis)
importFrom(stats,pnbinom)
importFrom(stats,pnorm)
importFrom(stats,poisson)
importFrom(stats,ppois)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,profile)
importFrom(stats,pt)
importFrom(stats,qchisq)
importFrom(stats,qnbinom)
importFrom(stats,qnorm)
importFrom(stats,reformulate)
importFrom(stats,residuals)
importFrom(stats,rnbinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,simulate)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weights)
importFrom(stats,xtabs)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,packageVersion)
useDynLib(glmmTMB)
