% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reformulas.R
\name{findReTrmClasses}
\alias{findReTrmClasses}
\title{list of specials -- taken from enum.R}
\usage{
findReTrmClasses()
}
\description{
list of specials -- taken from enum.R
}
