% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_autopar}
\alias{get_autopar}
\title{retrieve current value of TMB autopar setting}
\usage{
get_autopar()
}
\description{
retrieve current value of TMB autopar setting
}
