% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmmTMB.R
\name{.collectDuplicates}
\alias{.collectDuplicates}
\title{collapse duplicated observations}
\usage{
.collectDuplicates(data.tmb)
}
\description{
collapse duplicated observations
}
\keyword{internal}
